% Copyright (c) 2016, Carlos Ciller 
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
% 
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in
%       the documentation and/or other materials provided with the distribution
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.

function [ Adjacencies, Weights ] = createConnectivity( mode, volumeSize )
%CREATECONNECTIVITY Summary of this function goes here
%   Detailed explanation goes here

    x = volumeSize(1);
    y = volumeSize(2);
    z = volumeSize(3);
    N = x*y*z;
    
    xy = x * y;
    
    id = speye(N,N);
    
    %default 6 connections
    adjacencyMatrix = ...
        shiftmatrixSparse(id, 1, [0, 1], 0) + ...
        shiftmatrixSparse(id, 1, [0, y], 0) + ...
        shiftmatrixSparse(id, 1, [0, xy], 0);
    
    adjacencyMatrix = adjacencyMatrix + adjacencyMatrix';
    
    %additional diagonal connections
    sqrt2 = sqrt(2);
    sqrt3 = sqrt(3);
    
    if (strcmp(mode, '26') == 1)
        diagonalAdjacencies = ...
            sqrt2*shiftmatrixSparse(id, 1, [0, -1-y], 0) + ...
            sqrt2*shiftmatrixSparse(id, 1, [0, -1+y], 0) + ...
            sqrt2*shiftmatrixSparse(id, 1, [0, +1-y], 0) + ...
            sqrt2*shiftmatrixSparse(id, 1, [0, +1+y], 0);
        diagonalAdjacencies = diagonalAdjacencies + ...
            sqrt3*shiftmatrixSparse(id, 1, [0, -1-y + xy], 0) + ...
            sqrt3*shiftmatrixSparse(id, 1, [0, -1+y + xy], 0) + ...
            sqrt3*shiftmatrixSparse(id, 1, [0, +1-y + xy], 0) + ...
            sqrt3*shiftmatrixSparse(id, 1, [0, +1+y + xy], 0) + ...
            sqrt2*shiftmatrixSparse(id, 1, [0, -y + xy], 0) + ...
            sqrt2*shiftmatrixSparse(id, 1, [0, +y + xy], 0) + ...
            sqrt2*shiftmatrixSparse(id, 1, [0, -1 + xy], 0) + ...
            sqrt2*shiftmatrixSparse(id, 1, [0, +1 + xy], 0);
        diagonalAdjacencies = diagonalAdjacencies + ...
            sqrt3*shiftmatrixSparse(id, 1, [0, -1-y - xy], 0) + ...
            sqrt3*shiftmatrixSparse(id, 1, [0, -1+y - xy], 0) + ...
            sqrt3*shiftmatrixSparse(id, 1, [0, +1-y - xy], 0) + ...
            sqrt3*shiftmatrixSparse(id, 1, [0, +1+y - xy], 0) + ...
            sqrt2*shiftmatrixSparse(id, 1, [0, -y - xy], 0) + ...
            sqrt2*shiftmatrixSparse(id, 1, [0, +y - xy], 0) + ...
            sqrt2*shiftmatrixSparse(id, 1, [0, -1 - xy], 0) + ...
            sqrt2*shiftmatrixSparse(id, 1, [0, +1 - xy], 0);
        adjacencyMatrix = adjacencyMatrix + diagonalAdjacencies;
    end

%     imshow(full(adjacencyMatrix), []);
    
    [r,c,~]=find(adjacencyMatrix);

    % The Edges information is stored here... 
    Adjacencies=[c,r];
    Weights = adjacencyMatrix(sub2ind(size(adjacencyMatrix), c(:), r(:)));
end

