% Copyright (c) 2016, Carlos Ciller 
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
% 
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in
%       the documentation and/or other materials provided with the distribution
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.

function [ Adjacencies, Weights ] = createConnectivitySupervoxel( volume )

    x = size(volume,1);
    y = size(volume,2);
    z = size(volume,3);

    minimum = min(volume(:));
    maximum = max(volume(:));

    N = maximum - minimum + 1;

    adjacencyMatrix = speye(N,N);

    for i = 1:(x-1)
      for j = 1:(y-1)
        for k = 1:(z-1)
          ijk = volume(i,j,k)+1;
          i1jk = volume(i+1,j,k)+1;
          ij1k = volume(i,j+1,k)+1;
          ijk1 = volume(i,j,k+1)+1;

          if ijk ~= i1jk
            adjacencyMatrix(ijk, i1jk) = 1;
            adjacencyMatrix(i1jk, ijk) = 1;
          end
          if ijk ~= ij1k
            adjacencyMatrix(ijk, ij1k) = 1;
            adjacencyMatrix(ij1k, ijk) = 1;
          end
          if ijk ~= ijk1
            adjacencyMatrix(ijk, ijk1) = 1;
            adjacencyMatrix(ijk1, ijk) = 1;
          end
        end
      end
    end
    
    [r,c,~]=find(adjacencyMatrix);

    % The Edges information is stored here...
    Adjacencies=[c,r];
    Weights = ones(size(c,1), 1)';
end
