% Copyright (c) 2016, Carlos Ciller 
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
% 
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in
%       the documentation and/or other materials provided with the distribution
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.

function [flow,labels] = graphcutMRI(alpha, lambda, volume_intensity_T1, volume_intensity_T2, T1_tumor_std, T2_tumor_std,  votes, normalization_max, show_verbose)
    intensity_T1 = double(volume_intensity_T1);
    intensity_T2 = double(volume_intensity_T2);

    [v_height, v_width, v_depth] = size(votes);

    intensity_T1 = reshape(intensity_T1,1,[]);
    intensity_T2 = reshape(intensity_T2,1,[]);

    standard_deviation_data_T1 =  T1_tumor_std;
    standard_deviation_data_T2 = T2_tumor_std;

    votes = double(reshape(votes, 1, []));
    %normalize votes to 0..1
    %votes = votes ./ normalization_max;%% divide with number of trees
    
    % Carlos MODIFICATION
    votes = votes ./ max(votes);%% divide with number of trees

    if (min(votes(:)) < 0) || (max(votes(:)) > 1)
      throw(MException('graphcutMRI:wrongnormalization','Votes are not normalized. Possibly wrong number of trees.'));
    end

%     disp('building graph');

    %% Construct graph
    mode = 6;
    tic;
    [Adjacencies, Weights] = createConnectivity(int2str(mode), size(volume_intensity_T2));
    toc

    % Variation ND Paper (Boykov) for B_p-q (Boundary similarity)
%     alpha   =   0.5; % Weighting similarity T2
    beta = 1.0-alpha; % Weighting similarity T1
    V_T1 = exp(-((intensity_T1(Adjacencies(:,1))-intensity_T1(Adjacencies(:,2))).^2)./(2*standard_deviation_data_T1^2))+eps; % T1
    V_T2 = exp(-((intensity_T2(Adjacencies(:,1))-intensity_T2(Adjacencies(:,2))).^2)./(2*standard_deviation_data_T2^2))+eps; % T2
    V = ((alpha * V_T1 + beta * V_T2)./Weights');

    N = numel(volume_intensity_T2);
    A = sparse(Adjacencies(:,1), Adjacencies(:,2), V', N, N, mode * N);

    %% Weighting factor - Graph Cut

    % Lambda factor to multiply probability map (not normalized)
    labels_weight_healthy = double(zeros(size(intensity_T2, 2),1));
    labels_weight_tumor = double(zeros(size(intensity_T2, 2),1));

    % tumor labels
    labels_weight_tumor(:) = -lambda * log(eps+double(votes(:)));% negative-log-likelihood
    % healthy labels
    labels_weight_healthy(:) = -lambda * log(eps+double(1-votes(:))); %negative-log-likelihood

    labels_weight = [labels_weight_tumor, labels_weight_healthy];

    T=sparse(labels_weight);

%     disp('calculating maximum flow');

    tic;
    [flow,labels] = maxflow(A,T);
    toc
    labels = reshape(labels,[v_height v_width v_depth]);

    votes = reshape(votes,[v_height v_width v_depth]);
  %%

    if(show_verbose)
        colormap gray;

%         intensity_T2 = reshape(intensity_T2, [v_height v_width v_depth]);

        labels_weight_reshaped1 = reshape(full(T(:,1)), [v_height v_width v_depth]);
        labels_weight_reshaped2 = reshape(full(T(:,2)), [v_height v_width v_depth]);

        for i=v_depth/2
            subplot(2,2,1);
            imshow(squeeze(votes(:,:,i)), []); title(strcat('votes, lambda: ', int2str(lambda)));
            subplot(2,2,2);
            imshow(squeeze(labels(:,:,i)), []); title('labels');
%             imshow(imfuse(squeeze(labels_weight_reshaped1(:,:,i)), squeeze(labels_weight_reshaped2(:,:,i)))); title('both');
            subplot(2,2,3);
%             imagesc(squeeze(intensity_T2(:,:,i))); title('Intensity T2');
            imshow(squeeze(labels_weight_reshaped1(:,:,i)), []); title('healthy');
            subplot(2,2,4);
            imshow(squeeze(labels_weight_reshaped2(:,:,i)), []); title('tumor');
            drawnow;
%             pause;

        end
    end

end
