% Copyright (c) 2016, Carlos Ciller 
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
% 
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in
%       the documentation and/or other materials provided with the distribution
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.


% Define the input parameters here
data_folder = '../../2_PlosOne_RF_CNN_Experiments/Sub_Dataset/Results/';
ground_truth_folder = '../../2_PlosOne_RF_CNN_Experiments/GroundTruth/';
original_data_folder = '../../2_PlosOne_RF_CNN_Experiments/GroundTruth/';
output_folder = '../../2_PlosOne_RF_CNN_Experiments/Sub_Dataset/output/';

alpha = 0.1:0.1:0.3;
lambda = 0.3:0.1:0.8;

normalization_max = 200;
supervoxel = false; %do not change this

% Library paths
addNecessaryPaths();

% Possible scenarios and patient names
scenarios = loadScenarios(data_folder);
patients = loadScenarios(ground_truth_folder);

% Compute the mean and std
[t1_tumor_std, t2_tumor_std, T1_map, T2_map, GT_map] = computeStd(original_data_folder, ground_truth_folder, patients);

% Graph cut
graphCutRange_FundusPrior(supervoxel, data_folder, scenarios, patients, t1_tumor_std, t2_tumor_std, alpha, lambda, T1_map, T2_map, normalization_max);

% Compute APP (Accuracy per pixel), VOC Score, Dice
[app, dices, voc] = analyseDice(alpha, lambda, scenarios, patients, ground_truth_folder, data_folder);

% Result is stored

save(fullfile(output_folder, 'FP_grid_app.mat'), 'app');
save(fullfile(output_folder, 'FP_grid_dices.mat'), 'dices');
save(fullfile(output_folder, 'FP_grid_voc.mat'), 'voc');
save(fullfile(output_folder, 'FP_grid_scenarios.mat'), 'scenarios');
save(fullfile(output_folder, 'FP_grid_patients.mat'), 'patients');


% Result is displayed
cleanDices = dices;
cleanDices(isnan(cleanDices)) = 0;
cleanDices = squeeze(mean(cleanDices,3));
cleanDices = squeeze(mean(cleanDices,3));
