% Copyright (c) 2016, Carlos Ciller 
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
% 
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in
%       the documentation and/or other materials provided with the distribution
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.

function [app, dices, voc] = analyseDice(alphaRange, lambdaRange, scenarios, patients, groundTruthFolder, dataFolder)
  % Compute APP (Accuracy per pixel), VOC Score, Dice
  nPatients = numel(patients);
  nAlphas = numel(alphaRange);
  nLambdas = numel(lambdaRange);
  app = zeros(nAlphas, nLambdas, numel(scenarios), nPatients);
  dices = zeros(nAlphas, nLambdas, numel(scenarios), nPatients);
  voc = zeros(nAlphas, nLambdas, numel(scenarios), nPatients);

  for alpha = 1:nAlphas
      for lambda = 1:nLambdas
          for scenarioIndex = 1:numel(scenarios)
              display(scenarios{scenarioIndex}.name);
              scenario = scenarios{scenarioIndex};

              for patientIndex = 1:numel(patients)
                  patient = patients{patientIndex};
                  display(patient.name);

                  tmp = load_untouch_nii(fullfile(groundTruthFolder, patient.name, 'GT.nii'));
                  GT = tmp.img;

                  GT(GT>0) = 1;

                  tmp = load_untouch_nii(fullfile(dataFolder, scenario.name, ...
                      patient.name, strcat('graphcut', num2str(alphaRange(alpha)), '-',num2str(lambdaRange(lambda)),'.nii')));
                  segmentation = tmp.img;

                  GT = GT(:);
                  segmentation = segmentation(:);

                  tp = sum(GT(segmentation > 0) > 0);
                  tn = sum(GT(segmentation < 1) < 1);

                  fp = sum(GT(segmentation > 0) < 1);
                  fn = sum(GT(segmentation < 1) > 0);

                  app(alpha, lambda, scenarioIndex, patientIndex) = (tp+tn) / (tp + tn + fp + fn);
                  voc(alpha, lambda, scenarioIndex, patientIndex) = tp / (tp + fp + fn);
                  dices(alpha, lambda, scenarioIndex, patientIndex) = diceCoefficient(GT(:), segmentation(:));
              end
          end
      end
  end
