% Copyright (c) 2016, Carlos Ciller 
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
% 
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in
%       the documentation and/or other materials provided with the distribution
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.

function [t1_tumor_std, t2_tumor_std, T1_map, T2_map, GT_map] = computeStd(originalDataFolder, groundTruthFolder, patients)
%COMPUTESTD Summary of this function goes here
%   Detailed explanation goes here
    T1_map = containers.Map;
    T2_map = containers.Map;

    GT_map = containers.Map;

    tumor_t1_values = {};
    tumor_t2_values = {};
    for j = 1:numel(patients)
        patient = patients{j};
        display(patient.name);

        tmp = load_untouch_nii(strcat(originalDataFolder, patient.name, '/Patient_T1.nii'));
        T1_map(patient.name) = tmp.img;
        tmp = load_untouch_nii(strcat(originalDataFolder, patient.name, '/Patient_T2.nii'));
        T2_map(patient.name) = tmp.img;

        tmp = load_untouch_nii(strcat(groundTruthFolder, patient.name, '/GT.nii'));
        GT_map(patient.name) = tmp.img > 0;

        T1 = T1_map(patient.name);
        T1 = T1(:);
        T2 = T2_map(patient.name);
        T2 = T2(:);
        GT = GT_map(patient.name);

        tumor_t1_values{j} = (T1(GT(:)>0))';
        tumor_t2_values{j} = (T2(GT(:)>0))';
    end
    tumor_t1_values = cat(2, tumor_t1_values{:});
    tumor_t2_values = cat(2, tumor_t2_values{:});

    t1_tumor_std = std(double(tumor_t1_values));
    t2_tumor_std = std(double(tumor_t2_values));
end

