% Copyright (c) 2016, Carlos Ciller 
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
% 
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in
%       the documentation and/or other materials provided with the distribution
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.

function [gridDices] = evaluateGridSearch(gridScenario, folder, groundTruthFolder, patients, alphaRange, lambdaRange)
%EVALUATEGRIDSEARCH Summary of this function goes here
%   Detailed explanation goes here
    gridDices = zeros(numel(patients), numel(alphaRange), numel(lambdaRange));

    for alphaIdx = 1:numel(alphaRange)
        alpha = alphaRange(alphaIdx);
        for lambdaIdx = 1:numel(lambdaRange)
            lambda = lambdaRange(lambdaIdx);

            for patientIdx = 1:numel(patients)
                patient = patients{patientIdx};

                tmp = load_untouch_nii(strcat(groundTruthFolder, patient.name, '/GTVT1.nii'));
                GTVT1 = tmp.img;
                tmp = load_untouch_nii(strcat(groundTruthFolder, patient.name, '/GTVT2.nii'));
                GTVT2 = tmp.img;

                GT = (GTVT1 + GTVT2);

                if exist('RetDetT1.nii', 'file')
                    tmp = load_untouch_nii(strcat(groundTruthFolder, patient.name, '/RetDetT1.nii'));
                    GT = GT + tmp.img;
                    tmp = load_untouch_nii(strcat(groundTruthFolder, patient.name, '/RetDetT2.nii'));
                    GT = GT + tmp.img;
                end

                GT(GT>0) = 1;

                votesFilename = getVotesFileName(gridScenario);

                file = dir(strcat(folder, gridScenario, '/', patient.name, '/RF', votesFilename));

                file = file(1);
                loadFile = strcat(folder, gridScenario, '/', patient.name, '/', file.name(1:end-4), '_graphcut', num2str(alpha), '-', num2str(lambda), '.nii');
                tmp = load_untouch_nii(loadFile);
                segmentation = tmp.img;

                GT = GT(:);
                segmentation = segmentation(:);            

                gridDices(patientIdx, alphaIdx, lambdaIdx) = diceCoefficient(GT(:), segmentation(:));
            end
        end
    end
end

