% Copyright (c) 2016, Carlos Ciller 
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
% 
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in
%       the documentation and/or other materials provided with the distribution
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.

function [] = graphCutRange(is_supervoxel, folder, scenarios, patients, t1_tumor_std, t2_tumor_std, alphaRange, lambdaRange, T1_map, T2_map, normalization_max)
%GRAPHCUTRANGE Summary of this function goes here
%   Detailed explanation goes here

    for i = 1:numel(scenarios)
        scenario = scenarios{i};
        display(scenarios{i}.name);

        for alphaIdx = 1:numel(alphaRange)
            alpha = alphaRange(alphaIdx);
            for lambdaIdx = 1:numel(lambdaRange)
                lambda = lambdaRange(lambdaIdx);
                for j = 1:numel(patients)
                    patient = patients{j};

                    T1 = T1_map(patient.name);
                    T2 = T2_map(patient.name);

                    votesFilename = getVotesFileName_FundusPrior(scenario.name);

                    file = dir(strcat(folder, scenario.name, '/', patient.name, '/', votesFilename));
                    strcat(folder, scenario.name, '/', patient.name, '/', votesFilename)

                    file = file(1);

                    votes = load_untouch_nii(fullfile(folder, scenario.name, '/', patient.name, '/', file.name));

                    if (is_supervoxel)

                        sv_ids = load_untouch_nii(fullfile(folder, scenario.name, patient.name, 'Index_SVOX_T2.nii'));

                        [~, segmentation] = graphcutMRISuper(alpha, lambda, T1, T2, t1_tumor_std, t2_tumor_std, votes.img, sv_ids.img, normalization_max);
                    else
                        [~, segmentation] = graphcutMRI(alpha, lambda, T1, T2, t1_tumor_std, t2_tumor_std, votes.img, normalization_max, false);
                    end
                    votes.img = segmentation;

                    save_untouch_nii(votes, fullfile(folder, scenario.name, patient.name, strcat('graphcut', num2str(alpha),'-',num2str(lambda),'.nii')));
                end
            end
        end
    end

end
